import java.awt.Color;

public class ZigZagFish extends Fish
{
  private boolean willZigRight;

    // true indicates the next move should be forward to the right
    // false indicates the next move should be forward to the left

  public ZigZagFish(Environment env, Location loc)
  {
    super(env, loc, env.randomDirection(), Color.yellow);
    willZigRight = true;
  }

  public ZigZagFish(Environment env, Location loc, Direction dir)
  {
    super(env, loc, dir, Color.yellow);
    willZigRight = true;
  }
        
  public ZigZagFish(Environment env, Location loc, Direction dir, Color col)
  {
    super(env, loc, dir, col);
    willZigRight = true;
  }

  protected void generateChild(Location loc)
  {
    ZigZagFish child = new ZigZagFish(environment(), loc,
                                  environment().randomDirection(), color());
  }

  /*** Part (a) ***/

  // returns the forward diagonal cell to the left or right of this fish
  // (depending on willZigRight) if that cell is empty;
  // otherwise, returns this fish's current location
  // postcondition: the state of this ZigZagFish is unchanged
  protected Location nextLocation()
  {
    Environment env = environment();

    Location nextLoc = env.getNeighbor(location(), direction());
    if (willZigRight)
      nextLoc = env.getNeighbor(nextLoc, direction().toRight());
    else
      nextLoc = env.getNeighbor(nextLoc, direction().toLeft());

    if (env.isEmpty(nextLoc))
      return nextLoc;

    return location();
  }

  /*** Part (b) ***/

  // moves this ZigZagFish diagonally (as specified in nextLocation) if
  // possible; otherwise, reverses direction without moving;
  // after a diagonal move, willZigRight is updated
  protected void move()
  {
    Location nextLoc = nextLocation();

    if (!nextLoc.equals(location()))
    {
      changeLocation(nextLoc);
      willZigRight = !willZigRight;
    }
    else
      changeDirection(direction().reverse());
  }
}

