import java.util.*;

public class PostalCodeDB
{
  /*** Part (a) ***/

  // The running times are O(1), O(1), because uses HashMap and HashSet
  // and all of the operations used on them in this code are O(1)

  private Map codeToCityMap; // each key is a postal code, its
                             // associated value is a set of cities

  /*** Part (b) ***/

  // additinal instance variable(s)

  private Map cityToCodeMap; // each key is a city, its
                             // associated value is a set of postal codes

  // constructs an empty database
  public PostalCodeDB()
  {
    codeToCityMap = new HashMap();

   /*** Part (b) ***/

    // initialization of additional instance variable(s)

    cityToCodeMap = new TreeMap();
  }

  // returns the set of cities associated with postalCode if that
  // code is in the database, otherwise returns null
  public Set getCitiesForCode(String code)
  {
    return (Set)codeToCityMap.get(code);
  }

  // returns the set of cities associated with postalCode if that
  // code is in the database, otherwise returns null
  public Set getCodesForCity(String city)
  {
    return (Set)cityToCodeMap.get(city);
  }

  // adds a new city, code pair to this database
  public void addCityCodePair(String code, String city)
  {
    Set citySet = getCitiesForCode(code);
    if (citySet == null)
    {
      citySet = new HashSet();
      codeToCityMap.put(code, citySet);
    }
    citySet.add(city);

    /*** Part (b) ***/

    Set codeSet = getCodesForCity(city);
    if (codeSet == null)
    {
      codeSet = new HashSet();
      cityToCodeMap.put(city, codeSet);
    }
    codeSet.add(code);
  }

  /*** Part (c) ***/

  // The running time for addCityCodePair is O(log N)
  // in the worst case because this
  // method works with a HashMap, TreeMap and HashSet;
  // HashMap's get and put operations are O(1),
  // TreeMap's get and put operations are O(log N),
  // HashSet's add method is O(1)
  //
  // The running time for printAllCities traverses a tree,
  // is no worse than O(N) because the number of nodes is <= N.

  /*** Part (d) ***/

  // prints an alphabetical list of all cities in this database
  // postcondition: the state of the data structure is not changed
  public void printAllCities()
  {
    Iterator it = cityToCodeMap.keySet().iterator();
    while(it.hasNext())
      System.out.println(it.next());

/* Or, simply:
    System.out.println(cityToCodeMap.keySet());
*/
  }
}
